@@ -9,7 +9,7 @@ Python 3 socket module.
 from __future__ import absolute_import
 import io
 import os
-import sys
+
 
 from gevent import _socketcommon
 from gevent._util import copy_globals
@@ -51,7 +51,7 @@ class _closedsocket(object):
 
     detach = fileno
 
-    def _dummy(*args, **kwargs): # pylint:disable=no-method-argument,unused-argument
+    def _dummy(*args, **kwargs): # pylint:disable=no-method-argument,unused-argument,no-self-argument
         raise OSError(EBADF, 'Bad file descriptor')
     # All _delegate_methods must also be initialized here.
     send = recv = recv_into = sendto = recvfrom = recvfrom_into = _dummy
@@ -110,31 +110,21 @@ class socket(_socketcommon.SocketMixin):
     # don't subclass it. This lets code that needs the raw _sock (not tied to the hub)
     # get it. This shows up in tests like test__example_udp_server.
 
-    if sys.version_info[:2] < (3, 7):
-        def __init__(self, family=AF_INET, type=SOCK_STREAM, proto=0, fileno=None):
-            super().__init__()
-            self._closed = False
-            self._sock = self._gevent_sock_class(family, type, proto, fileno)
-            self.timeout = None
-            self.__init_common()
-    else:
-        # In 3.7, socket changed to auto-detecting family, type, and proto
-        # when given a fileno.
-        def __init__(self, family=-1, type=-1, proto=-1, fileno=None):
-            super().__init__()
-            self._closed = False
-            if fileno is None:
-                if family == -1:
-                    family = AF_INET
-                if type == -1:
-                    type = SOCK_STREAM
-                if proto == -1:
-                    proto = 0
-            self._sock = self._gevent_sock_class(family, type, proto, fileno)
-            self.timeout = None
-            self.__init_common()
+    # In 3.7, socket changed to auto-detecting family, type, and proto
+    # when given a fileno.
+    def __init__(self, family=-1, type=-1, proto=-1, fileno=None):
+        super().__init__()
+        self._closed = False
+        if fileno is None:
+            if family == -1:
+                family = AddressFamily.AF_INET
+            if type == -1:
+                type = SOCK_STREAM
+            if proto == -1:
+                proto = 0
+        self._sock = self._gevent_sock_class(family, type, proto, fileno)
+        self.timeout = None
 
-    def __init_common(self):
         self._io_refs = 0
         _socket.socket.setblocking(self._sock, False)
         fileno = _socket.socket.fileno(self._sock)
@@ -321,6 +311,7 @@ class socket(_socketcommon.SocketMixin):
         # Break any reference to the loop.io objects. Our fileno,
         # which they were tied to, is about to be free to be reused, so these
         # objects are no longer functional.
+        # pylint:disable-next=superfluous-parens
         self._drop_events_and_close(closefd=(reason == 'closed'))
 
         self._sock = _closedsocket(family, type, proto, fileno, reason)
@@ -538,6 +529,77 @@ if hasattr(_socket.socket, "share"):
     __implements__.append('fromshare')
 
 
+def _fallback_socketpair(family=AF_INET, type=SOCK_STREAM, proto=0):
+    # We originally used https://gist.github.com/4325783, by Geert Jansen. (Public domain.)
+    # We took it from 3.6 release, confirmed unchanged in 3.7 and
+    # 3.8a1. Expected to be used only on Win. Added to Win/3.5.
+    # It is always available as `socket._fallback_socketpair` from at least 3.9,
+    # We would like to stop carrying around our own implementation, but
+    # using _fallback_socketpair directly would only work if we are monkey patched.
+
+    # Current version taken from 3.13rc2
+
+    # PyPy doesn't name its fallback `_fallback_socketpair`, it uses
+    # an older copy of socket.py.
+    _LOCALHOST = '127.0.0.1'
+    _LOCALHOST_V6 = '::1'
+
+    if family == AF_INET:
+        host = _LOCALHOST
+    elif family == AF_INET6:
+        host = _LOCALHOST_V6
+    else:
+        raise ValueError("Only AF_INET and AF_INET6 socket address families "
+                         "are supported")
+    if type != SOCK_STREAM:
+        raise ValueError("Only SOCK_STREAM socket type is supported")
+    if proto != 0:
+        raise ValueError("Only protocol zero is supported")
+
+    # We create a connected TCP socket. Note the trick with
+    # setblocking(False) that prevents us from having to create a thread.
+    lsock = socket(family, type, proto)
+    try:
+        lsock.bind((host, 0))
+        lsock.listen()
+        # On IPv6, ignore flow_info and scope_id
+        addr, port = lsock.getsockname()[:2]
+        csock = socket(family, type, proto)
+        try:
+            csock.setblocking(False)
+            try:
+                csock.connect((addr, port))
+            except (BlockingIOError, InterruptedError):
+                pass
+            csock.setblocking(True)
+            ssock, _ = lsock.accept()
+        except:
+            csock.close()
+            raise
+    finally:
+        lsock.close()
+
+    # Authenticating avoids using a connection from something else
+    # able to connect to {host}:{port} instead of us.
+    # We expect only AF_INET and AF_INET6 families.
+    try:
+        if (
+            ssock.getsockname() != csock.getpeername()
+            or csock.getsockname() != ssock.getpeername()
+        ):
+            raise ConnectionError("Unexpected peer connection")
+    except:
+        # getsockname() and getpeername() can fail
+        # if either socket isn't connected.
+        ssock.close()
+        csock.close()
+        raise
+
+    return (ssock, csock)
+
+if hasattr(__socket__, _fallback_socketpair.__name__):
+    __implements__.append(_fallback_socketpair.__name__)
+
 if hasattr(_socket, "socketpair"):
 
     def socketpair(family=None, type=SOCK_STREAM, proto=0):
@@ -563,53 +625,14 @@ if hasattr(_socket, "socketpair"):
         return a, b
 
 else: # pragma: no cover
-    # Origin: https://gist.github.com/4325783, by Geert Jansen.  Public domain.
+    socketpair = _fallback_socketpair
 
-    # gevent: taken from 3.6 release, confirmed unchanged in 3.7 and
-    # 3.8a1. Expected to be used only on Win. Added to Win/3.5
-
-    _LOCALHOST = '127.0.0.1'
-    _LOCALHOST_V6 = '::1'
-
-    def socketpair(family=AF_INET, type=SOCK_STREAM, proto=0):
-        if family == AF_INET:
-            host = _LOCALHOST
-        elif family == AF_INET6:
-            host = _LOCALHOST_V6
-        else:
-            raise ValueError("Only AF_INET and AF_INET6 socket address families "
-                             "are supported")
-        if type != SOCK_STREAM:
-            raise ValueError("Only SOCK_STREAM socket type is supported")
-        if proto != 0:
-            raise ValueError("Only protocol zero is supported")
-
-        # We create a connected TCP socket. Note the trick with
-        # setblocking(False) that prevents us from having to create a thread.
-        lsock = socket(family, type, proto)
-        try:
-            lsock.bind((host, 0))
-            lsock.listen()
-            # On IPv6, ignore flow_info and scope_id
-            addr, port = lsock.getsockname()[:2]
-            csock = socket(family, type, proto)
-            try:
-                csock.setblocking(False)
-                try:
-                    csock.connect((addr, port))
-                except (BlockingIOError, InterruptedError):
-                    pass
-                csock.setblocking(True)
-                ssock, _ = lsock.accept()
-            except:
-                csock.close()
-                raise
-        finally:
-            lsock.close()
-        return (ssock, csock)
 
 
 __all__ = __implements__ + __extensions__ + __imports__
+if _fallback_socketpair.__name__ in __all__:
+    __all__.remove(_fallback_socketpair.__name__)
+
 __version_specific__ = (
     # Python 3.7b1+
     'close',
