@@ -11,6 +11,8 @@ Greenlet-local objects support the management of greenlet-local data.
 If you have data that you want to be local to a greenlet, simply create
 a greenlet-local object and use its attributes:
 
+  >>> import gevent
+  >>> from gevent.local import local
   >>> mydata = local()
   >>> mydata.number = 42
   >>> mydata.number
@@ -364,9 +366,9 @@ class local(object):
     """
     __slots__ = tuple(_local_attrs - {'__class__', '__cinit__'})
 
-    def __cinit__(self, *args, **kw):
+    def __cinit__(self, *args, **kw): # pylint:disable=bad-dunder-name
         if args or kw:
-            if type(self).__init__ == object.__init__:
+            if type(self).__init__ == object.__init__: # pylint:disable=comparison-with-callable
                 raise TypeError("Initialization arguments are not supported", args, kw)
         impl = _localimpl(args, kw, type(self), id(self))
         # pylint:disable=attribute-defined-outside-init
@@ -572,32 +574,35 @@ def _local_find_descriptors(self):
 # (e.g., on PyPy). So we set it at runtime. Cython
 # will raise an error if we're compiled.
 def __new__(cls, *args, **kw):
-    self = super(local, cls).__new__(cls)
+    self = super(local, cls).__new__(cls) # pylint:disable=no-value-for-parameter
     # We get the cls in *args for some reason
     # too when we do it this way....except on PyPy3, which does
     # not *unless* it's wrapped in a classmethod (which it is)
     self.__cinit__(*args[1:], **kw)
     return self
 
-try:
+if local.__module__ == 'gevent.local':
     # PyPy2/3 and CPython handle adding a __new__ to the class
     # in different ways. In CPython and PyPy3, it must be wrapped with classmethod;
-    # in PyPy2, it must not. In either case, the args that get passed to
+    # in PyPy2 < 7.3.3, it must not. In either case, the args that get passed to
     # it are stil wrong.
-    local.__new__ = 'None'
-except TypeError: # pragma: no cover
-    # Must be compiled
-    pass
-else:
-    from gevent._compat import PYPY
-    from gevent._compat import PY2
-    if PYPY and PY2:
-        local.__new__ = __new__
-    else:
-        local.__new__ = classmethod(__new__)
-
-    del PYPY
-    del PY2
+    #
+    # Prior to Python 3.10, Cython-compiled classes were immutable and
+    # raised a TypeError on assignment to __new__, and we relied on that
+    # to detect the compiled version; but that breaks in
+    # 3.10 as classes are now mutable. (See
+    # https://github.com/cython/cython/issues/4326).
+    #
+    # That's OK; post https://github.com/gevent/gevent/issues/1480, the Cython-compiled
+    # module has a different name than the pure-Python version and we can check for that.
+    # It's not as direct, but it works.
+    # So here we're not compiled
+    local.__new__ = classmethod(__new__)
+else: # pragma: no cover
+    # Make sure we revisit in case of changes to the (accelerator) module names.
+    if local.__module__ != 'gevent._gevent_clocal': # pylint:disable=else-if-used
+        raise AssertionError("Module names changed (local: %r; __name__: %r); revisit this code" % (
+            local.__module__, __name__) )
 
 _init()
 
