@@ -59,15 +59,18 @@ PLATFORM_SPECIFIC_SUFFIXES = ('2', '279', '3')
 if WIN:
     PLATFORM_SPECIFIC_SUFFIXES += ('posix',)
 
-PY2 = None
-PY3 = None
+PY2 = False # Never again
+PY3 = True
 PY35 = None
 PY36 = None
 PY37 = None
 PY38 = None
 PY39 = None
+PY39_EXACTLY = None
 PY310 = None
 PY311 = None
+PY312 = None
+PY313 = None
 
 NON_APPLICABLE_SUFFIXES = ()
 if sys.version_info[0] == 3:
@@ -85,34 +88,24 @@ if sys.version_info[0] == 3:
         PY38 = True
     if sys.version_info[1] >= 9:
         PY39 = True
+        if sys.version_info[:2] == (3, 9):
+            PY39_EXACTLY = True
     if sys.version_info[1] >= 10:
         PY310 = True
     if sys.version_info[1] >= 11:
         PY311 = True
+    if sys.version_info[1] >= 12:
+        PY312 = True
+    if sys.version_info[1] >= 13:
+        PY313 = True
+
 
-elif sys.version_info[0] == 2:
-    # Any python 2
-    PY3 = False
-    PY2 = True
-    NON_APPLICABLE_SUFFIXES += ('3',)
-    if (sys.version_info[1] < 7
-            or (sys.version_info[1] == 7 and sys.version_info[2] < 9)):
-        # Python 2, < 2.7.9
-        NON_APPLICABLE_SUFFIXES += ('279',)
 else: # pragma: no cover
     # Python 4?
     raise ImportError('Unsupported major python version')
 
 PYPY3 = PYPY and PY3
 
-PY27_ONLY = sys.version_info[0] == 2 and sys.version_info[1] == 7
-
-PYGTE279 = (
-    sys.version_info[0] == 2
-    and sys.version_info[1] >= 7
-    and sys.version_info[2] >= 9
-)
-
 if WIN:
     NON_APPLICABLE_SUFFIXES += ("posix",)
     # This is intimately tied to FileObjectPosix
@@ -129,7 +122,7 @@ RUNNING_ON_CI = RUNNING_ON_TRAVIS or RUNNING_ON_APPVEYOR
 RUNNING_ON_MANYLINUX = os.environ.get('GEVENT_MANYLINUX')
 # I'm not sure how to reliably auto-detect this, without
 # importing platform, something we don't want to do.
-RUNNING_ON_MUSLLINUX = os.environ.get('GEVENT_MANYLINUX_NAME') == 'musllinux'
+RUNNING_ON_MUSLLINUX = 'musllinux' in os.environ.get('GEVENT_MANYLINUX_NAME', '')
 
 if RUNNING_ON_APPVEYOR:
     # We can't exec corecext on appveyor if we haven't run setup.py in
@@ -182,36 +175,84 @@ def get_python_version():
 
     return version
 
-# XXX: In Python 3.10, distutils is deprecated and slated for removal in
-# 3.12. The suggestion is to use setuptools, but it only has LooseVersion
-# in an internal package and suggests using the new dependency of 'packaging'
+def _parse_version(ver_str):
+    try:
+        from packaging.version import Version
+        # InvalidVersion is a type of ValueError
+    except ImportError:
+        import warnings
+        warnings.warn('packaging.version not available; assuming no advanced Linux backends')
+        raise ValueError
+
+    try:
+        return Version(ver_str)
+    except ValueError:
+        import warnings
+        warnings.warn('Unable to parse version %s' % (ver_str,))
+        raise
+
+def _check_linux_version_at_least(major, minor, error_kind):
+    # pylint:disable=too-many-return-statements
+    # ^ Yeah, but this is the most linear and simple way to
+    # write this.
+    from platform import system
+    if system() != 'Linux':
+        return  False
+
+    from platform import release as _release
+    release = _release()
+    try:
+        # Linux versions like '6.8.0-1014-azure' cannot be parsed
+        # by packaging.version.Version, and distutils.LooseVersion, which
+        # did handle that, is deprecated. Neither module is guaranteed to be available
+        # anyway, so do the best we can manually.
+        ver_strings = (release or '0').split('.', 2)
+
+        if not ver_strings or int(ver_strings[0]) < major: # no way.
+            return False
+
+        if int(ver_strings[0]) > major: # Way newer!
+            return True
+
+        assert major == int(ver_strings[0]) # Exactly the major
+
+        if len(ver_strings) < 2: # no minor version, assume no
+            return False
+
+        if int(ver_strings[1]) < minor:
+            return False
+
+        assert int(ver_strings[1]) >= minor, (ver_strings[1], minor)
+        return True
+    except AssertionError:
+        raise
+    except Exception: # pylint:disable=broad-exception-caught
+        import warnings
+        warnings.warn('Unable to parse version %r; assuming no %s support' % (
+            release, error_kind
+        ))
+        return False
 
 def libev_supports_linux_aio():
     # libev requires kernel 4.19 or above to be able to support
     # linux AIO. It can still be compiled in, but will fail to create
     # the loop at runtime.
-    from distutils.version import LooseVersion # pylint:disable=deprecated-module
-    from platform import system
-    from platform import release
+    return _check_linux_version_at_least(4, 19, 'aio')
 
-    return system() == 'Linux' and LooseVersion(release() or '0') >= LooseVersion('4.19')
 
 def libev_supports_linux_iouring():
     # libev requires kernel XXX to be able to support linux io_uring.
     # It fails with the kernel in fedora rawhide (4.19.76) but
     # works (doesn't fail catastrophically when asked to create one)
     # with kernel 5.3.0 (Ubuntu Bionic)
-    from distutils.version import LooseVersion # pylint:disable=deprecated-module
-    from platform import system
-    from platform import release
+    return _check_linux_version_at_least(5, 3, 'iouring')
 
-    return system() == 'Linux' and LooseVersion(release() or '0') >= LooseVersion('5.3')
 
 def resolver_dnspython_available():
     # Try hard not to leave around junk we don't have to.
-    import pkg_resources
+    from importlib import metadata
     try:
-        pkg_resources.get_distribution('dnspython')
-    except pkg_resources.DistributionNotFound:
+        metadata.distribution('dnspython')
+    except metadata.PackageNotFoundError:
         return False
     return True
