@@ -1,4 +1,11 @@
-from __future__ import print_function, absolute_import, division
+"""
+.. caution::
+
+    This module imports `subprocess` and `threading`; this can
+    break monkey-patched unittests. Specifically,
+    ``test_threading.ThreadTests.test_import_from_another_thread``.
+"""
+
 import re
 import sys
 import os
@@ -260,7 +267,6 @@ class RunResult(object):
     value of True; otherwise, a boolean value of false.
 
     The integer value of this object is the command's exit code.
-
     """
 
     def __init__(self,
@@ -358,7 +364,7 @@ def _find_test_status(took, out):
         if m:
             skipped = _colorize('skipped', out[m.start():m.end()])
             skipped_count = int(out[m.start(1):m.end(1)])
-    status = status % (took, skipped)
+    status = status % (took, skipped) # pylint:disable=consider-using-augmented-assign
     if took > 10:
         status = _colorize('slow-test', status)
     return status, run_count, skipped_count
@@ -374,6 +380,7 @@ def run(command, **kwargs): # pylint:disable=too-many-locals
     quiet = kwargs.pop('quiet', QUIET)
     verbose = not quiet
     nested = kwargs.pop('nested', False)
+    allowed_return_codes = kwargs.pop('allowed_return_codes', ())
     if buffer_output:
         assert 'stdout' not in kwargs and 'stderr' not in kwargs, kwargs
         kwargs['stderr'] = subprocess.STDOUT
@@ -393,8 +400,10 @@ def run(command, **kwargs): # pylint:disable=too-many-locals
         kill(popen)
         assert popen.timer is None
 
-
-    failed = bool(result)
+    # We don't want to treat return codes that are allowed as failures,
+    # but we do want to log those specially. That's why we retain the distinction
+    # between ``failed`` and ``result`` (failed takes the allowed codes into account).
+    failed = bool(result) and result not in allowed_return_codes
     if out:
         out = out.strip()
         out = out if isinstance(out, str) else out.decode('utf-8', 'ignore')
@@ -405,11 +414,16 @@ def run(command, **kwargs): # pylint:disable=too-many-locals
         log('| %s\n%s', name, out)
     status, run_count, skipped_count = _find_test_status(duration, out)
     if result:
-        log('! %s [code %s] %s', name, result, status, color='error')
+        log('! %s [code %s] %s', name, result, status,
+            color='error' if failed else 'suboptimal-behaviour')
     elif not nested:
         log('- %s %s', name, status)
+
+    # For everything outside this function, we need to pretend that
+    # allowed codes are actually successes.
     return RunResult(
-        command, kwargs, result,
+        command, kwargs,
+        0 if result in allowed_return_codes else result,
         output=out, error=err,
         name=name,
         run_count=run_count,
